/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.area.field.shape;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.Action;
import javax.swing.ActionMap;
import nl.cloudfarming.client.model.Bean;
import nl.cloudfarming.client.model.CommonModelService;
import nl.cloudfarming.client.model.ShapeFile;
import org.openide.explorer.ExplorerManager;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Timon Veenstra
 */
public class ImportDataProvider extends CommonModelService {

    private ImportLayer importLayer;
    private final ShapeNodeKeys keys = new ShapeNodeKeys();
    private final List<Shape> shapes = new ArrayList<Shape>();

    public ImportDataProvider(ShapeFile plotFile) {
        importLayer = new ImportLayer(plotFile,this);
    }

    @Override
    protected final ExplorerManager initExplorer() {
        ExplorerManager manager = new ExplorerManager();
        manager.setRootContext(new ShapeFileNode(keys, importLayer));
        keys.addNotify();
        return manager;
    }

    /**
     * get the import layer containing the imported shapes
     *
     * @return import layer
     */
    public Node getLayerNode() {
        return getExplorerManager().getRootContext();
    }

    /**
     * get the list with managed shapes
     *
     * @return
     */
    public List<Shape> getShapes() {
        return Collections.unmodifiableList(shapes);
    }

    /**
     * add a shape to the list of managed shapes
     *
     * @param shape
     */
    public void addShape(Shape shape) {
        shapes.add(shape);
    }

    class ShapeFileNode extends AbstractNode{

        public ShapeFileNode(Children children, ImportLayer importLayer) {
            super(children, Lookups.fixed(importLayer));
            setDisplayName(importLayer.getName());
        }

        @Override
        public Action[] getActions(boolean context) {
            return importLayer.getActions();
        } 
    }
    
    class ShapeNodeKeys extends Children.Keys<Shape> {

        public ShapeNodeKeys() {
            super(false);
        }

        @Override
        protected void addNotify() {
            setKeys(getShapes());
        }
        
        @Override
        protected Node[] createNodes(Shape shape) {
            return new Node[]{new AbstractNode(Children.LEAF, Lookups.fixed(shape))};
        }
    }
}
